︠83bcf07c-acd8-4012-88f0-f62c049e70ea︠
r"""

real and complex numbers - As elements of the quaternion manifold H^1

This is all done with conjugates.

AUTHORS:
    -- Doug Sweetser (2009): Initial version.
    
EXAMPLES:
    Test the various conjugates.
    sage: q1 = q(1, 2, 3, 4)
    sage: q1.real()==q(1, 0, 0, 0)
    True
    sage: q1.complex_x()==q(1, 2, 0, 0)
    True
    sage: q1.complex_y()==q(1, 0, 3, 0)
    True
    sage: q1.complex_z()==q(1, 0, 0, 4)
    True
"""
#*****************************************************************************
#       Copyright (C) 2009 Doug Sweetser <sweetser@alum.mit.edu>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#                  http://www.gnu.org/licenses/
#*****************************************************************************
︡e0a551fc-db7d-4a75-8675-f3aceecbc20c︡︡
︠974ccda9-6115-4e61-aee8-ef3989753d10︠
class q(list):
    def __init__(self,t,x,y,z):
        self.t = t
        self.x = x
        self.y = y
        self.z = z
        list.__init__(self,[t,x,y,z])
    def __repr__(self):
        return "Quaternion tools."
    def show(self):
        print self.t, self.x, self.y, self.z
    def add(self, q2):
        return(q(self.t + q2.t, self.x + q2.x, self.y + q2.y, self.z + q2.z));
    def scalar_div(self, d):
        return(q(self.t/d, self.x/d, self.y/d, self.z/d));
    def scalar_x(self, d):
        return(q(self.t * d, self.x * d, self.y * d, self.z * d));
    def conj(self):
        return q(self.t, -self.x, -self.y, -self.z)
    def conj_1(self):
        return q(-self.t, self.x, -self.y, -self.z)   
    def conj_2(self):
        return q(-self.t, -self.x, self.y, -self.z)
    def conj_3(self):
        return q(-self.t, -self.x, -self.y, self.z)   
    def real(self):
        t = self.add(self.conj()).scalar_div(2)
        return q(t.t, t.x, t.y, t.z)
    def complex_x(self):
         tx =self.add(self).add(self.conj()).add(self.conj_1()).scalar_div(2)
         return q(tx.t, tx.x, tx.y, tx.z)
    def complex_y(self):
         ty = self.add(self).add(self.conj()).add(self.conj_2()).scalar_div(2)
         return q(ty.t, ty.x, ty.y, ty.z)
    def complex_z(self):
         tz = self.add(self).add(self.conj()).add(self.conj_3()).scalar_div(2)
         return q(tz.t, tz.x, tz.y, tz.z)
︡050da2cd-594c-4111-bdf6-c19a4e2125d7︡︡
︠497c0068-a5ff-47af-9488-b5b295730524︠
q1=q(1,2,3,4)
q1.real().show()
q1.complex_x().show()
q1.complex_y().show()
q1.complex_z().show()
︡12701713-1a1d-4ba3-88f1-1a30329d0856︡{"stdout": "1 0 0 0\n1 2 0 0\n1 0 3 0\n1 0 0 4"}︡
︠3472b6eb-42d7-4f7f-a82f-71fef2ec8f20︠
q(1, 2, 3, 4).complex_x()==q(1, 2, 0, 0)
︡daa04d70-1afa-4e95-b486-545d55b68a5d︡{"stdout": "True"}︡
︠b4e4c797-1704-4b9a-88ea-49d06be265a8︠
q(1, 2, 3, 4).complex_y()==q(1, 0, 3, 0)
︡08f2f3fe-4681-4d5b-8190-15a06c47c5e0︡{"stdout": "True"}︡
︠3c25a3c8-cfed-4bc1-9ece-8294a22bd1d7︠
q(1, 2, 3, 4).complex_z()==q(1, 0, 0, 4)
︡a9180ae9-e486-4bb9-8a34-8d300899a9b6︡{"stdout": "True"}︡
︠66d38a32-9547-4db6-9df9-e6e31c80a81c︠
r"""It is neat that the three familiar complex planes are created by doubling the input and adding in two conjugates before dividing by half.  This is how one can see the complex manifold C^1 is embedded in the quaternion manifold H^1.
"""
︡aaed187e-7f45-4c4f-9e3a-070eef2655c3︡︡
